local AddonProfilesComboBox = ZO_ComboBox:Subclass()
AddonProfiles.ComboBox = AddonProfilesComboBox

local PROFILE_SORT_KEYS = {
    sortGroup = { isNumeric = true, tiebreaker = "bareName" },
    bareName = { tiebreaker = "highlightName" },
    highlightName = {},
}


function AddonProfilesComboBox:New(container)
    local comboBox = ZO_ComboBox.New(self, container)
    comboBox:SetSortOrder(ZO_SORT_ORDER_UP, PROFILE_SORT_KEYS, "sortGroup")
    return comboBox
end


-- override
-- make entries callable to avoid the need to create closures in AddMenuItems
function AddonProfilesComboBox:CreateItemEntry(name, callback)
    local entry = {owner = self, name = name, callback = callback}
    return setmetatable(entry, self.ItemEntry)
end

local function ItemEntryOnSelect(entry)
    ZO_ComboBox_Base_ItemSelectedClickHelper(entry.owner, entry)
end

AddonProfilesComboBox.ItemEntry = {__call = ItemEntryOnSelect}


-- override
-- use AddCustomMenuItem instead of AddMenuItem and leverage callable entries
function AddonProfilesComboBox:AddMenuItems()
    for i = 1, #self.m_sortedItems do
        local item = self.m_sortedItems[i]
        AddCustomMenuItem(item.name, item, nil, self.m_font, self.m_normalColor,
                          self.m_highlightColor)
    end
end


-- new
function AddonProfilesComboBox:AddItem2(sortGroup, profileName, name, callback, selected)
    local item = self:CreateItemEntry(name, callback)
    item.sortGroup = sortGroup
    item.profileName = profileName
    -- strip color codes from name for proper sorting
    item.bareName = name:gsub("|c%x%x%x%x%x%x", ""):gsub("|r", "")
    if item.bareName ~= item.name then
        item.highlightName = item.name
    else
        -- highlight name for use in dialogs
        item.highlightName = ZO_HIGHLIGHT_TEXT:Colorize(item.bareName)
    end
    self:AddItem(item, ZO_COMBOBOX_SUPRESS_UPDATE)
    if selected then
        ZO_ComboBox_Base_ItemSelectedClickHelper(self, item, true)
    end
    return item
end


-- new
function AddonProfilesComboBox:FindItemByKey(key, value)
    for index, item in ipairs(self.m_sortedItems) do
        if item[key] == value then
            return item
        end
    end
end


-- new
function AddonProfilesComboBox:RemoveItemByKey(key, value)
    for index, item in ipairs(self.m_sortedItems) do
        if item[key] == value then
            table.remove(self.m_sortedItems, index)
            self:UpdateItems()
            return item
        end
    end
end


-- new
function AddonProfilesComboBox:SelectItemByKey(key, value, ignoreCallback)
    for index, item in ipairs(self.m_sortedItems) do
        if item[key] == value then
            ZO_ComboBox_Base_ItemSelectedClickHelper(self, item, ignoreCallback)
            return item
        end
    end
end


-- override bugged SetSortOrder, add sortKey parameter
function AddonProfilesComboBox:SetSortOrder(sortOrder, sortType, sortKey)
    self.m_sortOrder = sortOrder
    self.m_sortType = sortType or ZO_SORT_BY_NAME
    self.m_sortKey = sortKey or "name"
    self:UpdateItems()
end


-- override bugged UpdateItems, use sortKey if set
function AddonProfilesComboBox:UpdateItems()
    if self.m_sortsItems then
        -- cache sort parameters for faster access in compareItems
        local sortKey = self.m_sortKey or "name"
        local sortType = self.m_sortType
        local sortOrder = self.m_sortOrder
        local function compareItems(item1, item2)
            return ZO_TableOrderingFunction(item1, item2, sortKey, sortType, sortOrder)
        end
        table.sort(self.m_sortedItems, compareItems)
    end

    if self:IsDropdownVisible() then
        self:ShowDropdown()
    end
end
